@@ -54,8 +54,7 @@ def num_threads():
 def list_dir_recursively(root, array_of_files):
     for file in os.listdir(root):
         if (os.path.isfile(os.path.join(root, file))):
-            if (not file.startswith('USA_all')):
-                array_of_files.append(os.path.join(root, file))
+            array_of_files.append(os.path.join(root, file))
         else:
             list_dir_recursively(os.path.join(root, file), array_of_files)
 
@@ -166,7 +165,7 @@ def list_one_dir(client, bucket, prefix_in, recurse, array_of_files):
             for prefix in common_prefixes:
                 this_prefix = str(prefix['Prefix'])
                 # print('   ' + this_prefix)
-                if (bool(recurse) and this_prefix != None ):
+                if (bool(recurse) and this_prefix != None and not_dot_infinstor(this_prefix)):
                     list_one_dir(client, bucket, this_prefix, recurse, array_of_files)
 
 # returns parentdir (with no leading or trailing /) and filename
@@ -180,21 +179,33 @@ def get_parent_dir_and_fn(full_object_key):
             parentdir = parentdir + sep + comp
     return parentdir, components[len(components) - 1]
 
+def not_dot_infinstor(prefix):
+    components = prefix.rstrip('/').split(sep)
+    if (len(components) > 1 and components[len(components) - 1].startswith('.infinstor')):
+        return False
+    else:
+        return True
+
 # fills out dict_of_arrays_of_files with a dict of parentdir -> array_of_files_in_parent_dir
 def list_dir_by_dir(client, bucket, prefix_in, recurse, dict_of_arrays_of_files):
+    if (verbose == True):
+        print('list_dir_by_dir: Entered. bucket=' + bucket + ', prefix_in=' + prefix_in)
     paginator = client.get_paginator('list_objects_v2')
     page_iterator = paginator.paginate(Bucket=bucket, Prefix=prefix_in, Delimiter="/")
     for page in page_iterator:
 
-        # print('Files:')
+        if (verbose == True):
+            print('list_dir_by_dir: Files:')
         contents = page.get('Contents')
         if (contents != None):
-            # print('   ' + str(contents))
+            if (verbose == True):
+                print('list_dir_by_dir:    ' + str(contents))
             count = 0;
             for one_content in contents:
                 object_name = one_content['Key']
                 full_object_name = object_name
-                # print(full_object_name)
+                if (verbose == True):
+                    print('list_dir_by_dir: full_object_name=' + full_object_name)
                 parent_dir, filename = get_parent_dir_and_fn(object_name)
                 if (parent_dir in dict_of_arrays_of_files):
                     files_in_this_dir = dict_of_arrays_of_files[parent_dir]
@@ -206,13 +217,15 @@ def list_dir_by_dir(client, bucket, prefix_in, recurse, dict_of_arrays_of_files)
             if (count > 0):
                 print(str(count) + " files in " + prefix_in)
 
-        # print('Directories:')
+        if (verbose == True):
+            print('list_dir_by_dir: Directories:')
         common_prefixes = page.get('CommonPrefixes')
         if (common_prefixes != None):
             for prefix in common_prefixes:
                 this_prefix = str(prefix['Prefix'])
-                # print('   ' + this_prefix)
-                if (bool(recurse) and this_prefix != None ):
+                if (verbose == True):
+                    print('list_dir_by_dir:    ' + this_prefix)
+                if (bool(recurse) and this_prefix != None and not_dot_infinstor(this_prefix)):
                     list_dir_by_dir(client, bucket, this_prefix, recurse, dict_of_arrays_of_files)
 
 def info(msg):
@@ -229,10 +242,11 @@ def log_artifacts_recursively(temp_output_dir):
             log_artifact(filename, key)
 
 class download_task:
-    def __init__(self, command, bucketname, parentdir, filename):
+    def __init__(self, command, bucketname, parentdir, prefix_trunc, filename):
         self.command = command
         self.bucketname = bucketname
         self.parentdir = parentdir
+        self.prefix_trunc = prefix_trunc
         self.filename = filename
 
 def s3downloader(**kwargs):
@@ -273,10 +287,12 @@ def s3downloader(**kwargs):
         if (op == 'download'):
             bucketname = download_task.bucketname
             parentdir = download_task.parentdir
+            prefix_trunc = download_task.prefix_trunc
             filename = download_task.filename
             if (verbose == True):
                 info('s3downloader: Received download command. bucket='
                         + str(bucketname) + ', parentdir=' + str(parentdir)
+                        + ', prefix_trunc=' + str(prefix_trunc)
                         + ', filename=' + str(filename))
             if (parentdir == ''):
                 full_object_key = filename
@@ -329,7 +345,7 @@ def s3downloader(**kwargs):
                     #namespaced_infin_transform_one_object(bucketname, parentdir,\
                     #        filename, tmpf_name, **kwargs)
                     namespaced_infin_transform_one_object(tmpf_name, temp_output_dir,\
-                            parentdir, **kwargs)
+                            parentdir[len(prefix_trunc):], **kwargs)
                 except Exception as e1:
                     status_str = str(e1)
                     info("Error executing infin_transform_one_object_tmpfile: " + status_str)
@@ -384,15 +400,15 @@ def load_one_csv_from_bytearray(bts):
 
 # returns a pandas DataFrame with index 'YY-MM-dd HH:MM:SS bucketname/filename'
 # and one column named RawBytes that contains the raw bytes from the object
-def download_objects_inner(client, bucketname, parentdir, array_of_files, is_csv,\
+def download_objects_inner(client, bucketname, parentdir, prefix_trunc, array_of_files, is_csv,
         glb, tmpfile_dir, **kwargs):
     log_param("object_count", len(array_of_files))
 
     command_q = Queue(len(array_of_files) + num_threads())
     for onefile in array_of_files:
-        command_q.put(download_task('download', bucketname, parentdir, onefile))
+        command_q.put(download_task('download', bucketname, parentdir, prefix_trunc, onefile))
     for i in range(num_threads()):
-        command_q.put(download_task('quit', '', '', ''))
+        command_q.put(download_task('quit', '', '', '', ''))
 
     pipe_from_child = []
     processes = []
@@ -446,11 +462,11 @@ def download_objects_inner(client, bucketname, parentdir, array_of_files, is_csv
     return rv
 
 def actually_run_transformation(client, is_pandas_df, bucketname,\
-        prefix_in, is_csv, transformation_string, **kwargs):
+        prefix_in, prefix_trunc, is_csv, transformation_string, **kwargs):
     array_of_files = []
     list_one_dir(client, bucketname, prefix_in, True, array_of_files)
     print('actually_run_transformation: total number Of objects: ' + str(len(array_of_files)))
-    objects = download_objects_inner(client, bucketname, '', array_of_files, is_csv,\
+    objects = download_objects_inner(client, bucketname, '', prefix_trunc, array_of_files, is_csv,\
                 None, None, **kwargs)
     if (enable_infin_ast == True):
         transformAst = infin_ast.extract_transform(TRANSFORM_RAW_PD, src_str=transformation_string)
@@ -535,7 +551,7 @@ def run_xform_string_periodically(seconds, service_name, bucketname, prefix_in,
         time_spec = 'tm' + start_time.strftime("%Y%m%d%H%M%S")\
                 + '-tm' + end_time.strftime("%Y%m%d%H%M%S")
         actually_run_transformation(client, True, bucketname,\
-            prefix_in, False, transformation_string, **kwargs)
+            prefix_in, '', False, transformation_string, **kwargs)
 
 def new_dict_with_just_one(dict_in):
     one_dir = list(dict_in.keys())[0]
@@ -544,10 +560,10 @@ def new_dict_with_just_one(dict_in):
     return newdict
 
 def read_and_xform_one_object(client, bucketname,\
-        prefix_in, transform_string, **kwargs):
+        prefix_in, prefix_trunc, transform_string, **kwargs):
     if (verbose == True):
         print('read_and_xform_one_object: Entered. bucketname=' + str(bucketname)\
-            + ', prefix=' + str(prefix_in))
+            + ', prefix=' + str(prefix_in) + ', prefix_trunc=' + str(prefix_trunc))
     if (enable_infin_ast == True):
         transformAst = infin_ast.extract_transform(TRANSFORM_ONE_OBJ, src_str=transform_string)
         transformSrc = infin_ast.get_source(transformAst)
@@ -583,12 +599,12 @@ def read_and_xform_one_object(client, bucketname,\
         info("Execution of global statics for parentdir " + parentdir\
                 + " complete. status_str=" + status_str)
         tdir = tempfile.mkdtemp()
-        objects = download_objects_inner(client, bucketname, parentdir, array_of_files,\
+        objects = download_objects_inner(client, bucketname, parentdir, prefix_trunc, array_of_files,
                         False, glb, tdir, **kwargs)
         shutil.rmtree(tdir)
     return objects
 
-def read_and_xform_dir_by_dir(client, bucketname, prefix_in, transform_string, **kwargs):
+def read_and_xform_dir_by_dir(client, bucketname, prefix_in, prefix_trunc, transform_string, **kwargs):
     if (enable_infin_ast == True):
         transformAst = infin_ast.extract_transform(TRANSFORM_DIR_BY_DIR, src_str=transform_string)
         transformSrc = infin_ast.get_source(transformAst)
@@ -610,7 +626,7 @@ def read_and_xform_dir_by_dir(client, bucketname, prefix_in, transform_string, *
         info('Processing parentdir ' + parentdir + ' with ' + str(len(array_of_files))\
                 + ' objects, temp_input_dir=' + str(temp_input_dir)\
                 + ', temp_output_dir=' + str(temp_output_dir))
-        objects = download_objects_inner(client, bucketname, parentdir, array_of_files,\
+        objects = download_objects_inner(client, bucketname, parentdir, prefix_trunc, array_of_files,
                 False, None, temp_input_dir, **kwargs)
 
         if (enable_infin_ast == True):
@@ -643,29 +659,17 @@ def read_and_xform_dir_by_dir(client, bucketname, prefix_in, transform_string, *
 
         namespaced_infin_transform_fnx = namespaced_function(glb[xn], glb, None, True)
         try:
-            namespaced_infin_transform_fnx(temp_input_dir, temp_output_dir, parentdir, **kwargs)
+            namespaced_infin_transform_fnx(temp_input_dir, temp_output_dir, parentdir[len(prefix_trunc):], **kwargs)
         except Exception as e:
             status_str = str(e)
             info("Error executing " + xn + ", status=" + status_str)
         else:
-            info("Successfully executed transform " + xn + " for parentdir " + parentdir)
+            info("Successfully executed transform " + xn + " for parentdir " + parentdir[len(prefix_trunc):])
         finally:
             shutil.rmtree(temp_input_dir)
 
         try:
-            if (parentdir.startswith('mlflow-artifacts/')):
-                words = parentdir.split('/')
-                # e.g. mlflow-artifacts/jaganeds1/0/0-f95f0f7a8c2e47f6a9867cc04d0d4708/infinstor
-                if (len(words) >= 5):
-                    words = words[5:]
-                    dest_path = 'infinstor/'
-                    for word in words:
-                        dest_path += (word + '/')
-                else:
-                    dest_path = 'infinstor/' + parentdir
-            else:
-                dest_path = 'infinstor/' + parentdir
-
+            dest_path = 'infinstor/' + parentdir[len(prefix_trunc):]
             for one_output_file in os.listdir(temp_output_dir):
                 fq_local = os.path.join(temp_output_dir, one_output_file)
                 info("Logging artifact " + str(fq_local) + " to " + dest_path)
@@ -679,7 +683,7 @@ def read_and_xform_dir_by_dir(client, bucketname, prefix_in, transform_string, *
         finally:
             shutil.rmtree(temp_output_dir)
 
-def read_and_xform_all_objects(client, bucketname, prefix_in, transform_string, **kwargs):
+def read_and_xform_all_objects(client, bucketname, prefix_in, prefix_trunc, transform_string, **kwargs):
     if (enable_infin_ast == True):
         transformAst = infin_ast.extract_transform(TRANSFORM_DIR_BY_DIR, src_str=transform_string)
         transformSrc = infin_ast.get_source(transformAst)
@@ -699,14 +703,14 @@ def read_and_xform_all_objects(client, bucketname, prefix_in, transform_string,
     temp_output_dir = tempfile.mkdtemp()
 
     for parentdir in dict_of_arrays_of_files:
-        temp_input_dir = os.path.join(temp_input_dir_root, parentdir)
+        temp_input_dir = os.path.join(temp_input_dir_root, parentdir[len(prefix_trunc):])
         os.makedirs(temp_input_dir, mode=0o755, exist_ok=True)
         array_of_files = dict_of_arrays_of_files[parentdir]
         info('Processing parentdir ' + parentdir + ' with ' + str(len(array_of_files))\
                 + ' objects, temp_input_dir=' + str(temp_input_dir)\
                 + ', temp_output_dir=' + str(temp_output_dir))
-        objects = download_objects_inner(client, bucketname, parentdir, array_of_files,\
-                False, None, temp_input_dir, **kwargs)
+        objects = download_objects_inner(client, bucketname, parentdir, prefix_trunc,
+                array_of_files, False, None, temp_input_dir, **kwargs)
 
     if (enable_infin_ast == True):
         transformAst = infin_ast.extract_transform(TRANSFORM_ALL_OBJECTS,\
@@ -747,19 +751,7 @@ def read_and_xform_all_objects(client, bucketname, prefix_in, transform_string,
         shutil.rmtree(temp_input_dir)
 
     try:
-        if (prefix_in.startswith('mlflow-artifacts/')):
-            words = prefix_in.split('/')
-            # e.g. mlflow-artifacts/jaganeds1/0/0-f95f0f7a8c2e47f6a9867cc04d0d4708/infinstor
-            if (len(words) >= 5):
-                words = words[5:]
-                dest_path = 'infinstor/'
-                for word in words:
-                    dest_path += (word + '/')
-            else:
-                dest_path = 'infinstor/' + prefix_in
-        else:
-            dest_path = 'infinstor/' + prefix_in
-
+        dest_path = 'infinstor/' + prefix_in[len(prefix_trunc):]
         for one_output_file in os.listdir(temp_output_dir):
             fq_local = os.path.join(temp_output_dir, one_output_file)
             info("Logging artifact " + str(fq_local) + " to " + dest_path)
@@ -798,7 +790,7 @@ def get_mlflow_run_artifacts_info(run_id):
         raise ValueError('Error. Do not know how to deal with artifacts in scheme '\
                 + parse_result.scheme)
     bucketname = parse_result.netloc
-    prefix = parse_result.path.lstrip('/') + '/infinstor/'
+    prefix = parse_result.path.lstrip('/') + '/'
     return bucketname, prefix
 
 def generate_cache_key(input_data_spec, xformname):
@@ -855,6 +847,7 @@ def run_transform_inline(service_name, run_id, input_data_spec, xformname, **kwa
         endpoint = "https://" + input_data_spec['time_spec'] + ".s3proxy."\
                 + service_name + ".com:443/";
         prefix_in = input_data_spec['prefix']
+        prefix_trunc = ''
         bucketname = input_data_spec['bucketname']
         session = boto3.session.Session(profile_name='infinstor')
         client = session.client('s3', endpoint_url=endpoint)
@@ -862,11 +855,13 @@ def run_transform_inline(service_name, run_id, input_data_spec, xformname, **kwa
         endpoint = "https://lbl" + input_data_spec['label'] + ".s3proxy."\
                 + service_name + ".com:443/";
         prefix_in = '' # ignored for labels
+        prefix_trunc = '' # ignored for labels
         bucketname = '' # ignored for labels
         session = boto3.session.Session(profile_name='infinstor')
         client = session.client('s3', endpoint_url=endpoint)
     elif (input_data_spec['type'] == 'mlflow-run-artifacts'):
         bucketname, prefix_in = get_mlflow_run_artifacts_info(input_data_spec['run_id'])
+        prefix_trunc = prefix_in
         session = boto3.session.Session()
         client = session.client('s3')
     else:
@@ -879,43 +874,43 @@ def run_transform_inline(service_name, run_id, input_data_spec, xformname, **kwa
     log_param('run_id', run_id)
     log_param('cache_key', cache_key)
     log_param('input_data_spec', generate_input_data_spec_string(input_data_spec))
-    xobjects = run_transform_string_inline(client, bucketname, prefix_in,\
+    xobjects = run_transform_string_inline(client, bucketname, prefix_in, prefix_trunc,
                 transform_string, **kwargs)
     return xobjects
 
-def run_transform_string_inline(client, bucketname, prefix_in,\
+def run_transform_string_inline(client, bucketname, prefix_in, prefix_trunc,
         transform_string, **kwargs):
     if (look_for_transform(transform_string, TRANSFORM_RAW_PD)):
         print("Running transform " + str(TRANSFORM_RAW_PD))
         return actually_run_transformation(client, True,\
-                bucketname, prefix_in, False, transform_string,\
+                bucketname, prefix_in, prefix_trunc, False, transform_string,\
                 **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_RAW_DS)):
         print("Running transform " + str(TRANSFORM_RAW_DS))
         return actually_run_transformation(client, False,\
-                bucketname, prefix_in, False, transform_string,\
+                bucketname, prefix_in, prefix_trunc, False, transform_string,\
                 **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_CSV_PD)):
         print("Running transform " + str(TRANSFORM_CSV_PD))
         return actually_run_transformation(client, True,\
-                bucketname, prefix_in, True, transform_string,\
+                bucketname, prefix_in, prefix_trunc, True, transform_string,\
                 **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_CSV_DS)):
         print("Running transform " + str(TRANSFORM_CSV_DS))
         return actually_run_transformation(client, False,\
-                bucketname, prefix_in, True, transform_string,\
+                bucketname, prefix_in, prefix_trunc, True, transform_string,\
                 **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_ONE_OBJ)):
         print("Running transform " + str(TRANSFORM_ONE_OBJ))
-        return read_and_xform_one_object(client, bucketname, prefix_in,\
+        return read_and_xform_one_object(client, bucketname, prefix_in, prefix_trunc,
                 transform_string, **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_DIR_BY_DIR)):
         print("Running transform " + str(TRANSFORM_DIR_BY_DIR))
-        return read_and_xform_dir_by_dir(client, bucketname, prefix_in,\
+        return read_and_xform_dir_by_dir(client, bucketname, prefix_in, prefix_trunc,
                 transform_string, **kwargs)
     elif (look_for_transform(transform_string, TRANSFORM_ALL_OBJECTS)):
         print("Running transform " + str(TRANSFORM_ALL_OBJECTS))
-        return read_and_xform_all_objects(client, bucketname, prefix_in,\
+        return read_and_xform_all_objects(client, bucketname, prefix_in, prefix_trunc,
                 transform_string, **kwargs)
     else:
         raise Exception("Cannot find a known transform function");
@@ -1095,7 +1090,7 @@ def run_transforms(transforms):
 # and one column named RawBytes that contains the raw bytes from the object
 def download_objects(client, bucketname, array_of_files, is_csv):
     with start_run() as run:
-        return download_objects_inner(client, bucketname, '', array_of_files, is_csv,\
+        return download_objects_inner(client, bucketname, '', '', array_of_files, is_csv,\
                 None, None)
 
 def test_infin_transform(service_name, input_data_spec, **kwargs):
@@ -1105,20 +1100,9 @@ def test_infin_transform(service_name, input_data_spec, **kwargs):
         code_str = one_line # pick last cell's code
     # print(code_str)
 
-    mode = 0
-    dockerfile = ''
     clean_code = ''
     code_str_lines = code_str.splitlines()
     for one_line in code_str_lines:
-        if (one_line.lstrip().startswith('%infinstor-dockerfile')):
-            if (mode == 0):
-                mode = 1
-            else:
-                mode = 0
-            continue
-        if (mode == 1):
-            dockerfile += (one_line + '\n')
-            continue
         if (one_line == '%reset -f'):
             continue
         if (one_line == 'from infinstor import test_infin_transform # infinstor'):
@@ -1134,6 +1118,7 @@ def test_infin_transform(service_name, input_data_spec, **kwargs):
         endpoint = "https://" + input_data_spec['time_spec'] + ".s3proxy."\
                 + service_name + ".com:443/";
         prefix_in = input_data_spec['prefix']
+        prefix_trunc = ''
         bucketname = input_data_spec['bucketname']
         session = boto3.session.Session(profile_name='infinstor')
         client = session.client('s3', endpoint_url=endpoint)
@@ -1141,16 +1126,18 @@ def test_infin_transform(service_name, input_data_spec, **kwargs):
         endpoint = "https://lbl" + input_data_spec['label'] + ".s3proxy."\
                 + service_name + ".com:443/";
         prefix_in = '' # ignored for labels
+        prefix_trunc = '' # ignored for labels
         bucketname = '' # ignored for labels
         session = boto3.session.Session(profile_name='infinstor')
         client = session.client('s3', endpoint_url=endpoint)
     elif (input_data_spec['type'] == 'mlflow-run-artifacts'):
         bucketname, prefix_in = get_mlflow_run_artifacts_info(input_data_spec['run_id'])
+        prefix_trunc = prefix_in
         session = boto3.session.Session()
         client = session.client('s3')
     else:
         raise ValueError('Error. Unknown input_data_spec type ' + input_data_spec['type'])
 
     with start_run() as run:
-        run_transform_string_inline(client, bucketname, prefix_in, clean_code, **kwargs)
+        run_transform_string_inline(client, bucketname, prefix_in, prefix_trunc, clean_code, **kwargs)
         return run.info.run_id
